#!/usr/bin/env python3

import json
import re
import sys

def check_brackets(string):
    opened = 0
    for c in string:
        if c == '(':
            opened += 1
        elif c == ')':
            opened -= 1
    return string + max(0, opened) * ')'


def clean_up_string(string):
    wo_brackets = re.sub(r'\([^\)]*\)', '', string).replace("[", "").replace("]", "").replace(".", "")
    wo_brackets = wo_brackets.replace(' vnd ', ' und ')
    return wo_brackets


def clean_text(e):
    e = re.sub(r'<[^>]*>', r'', e)
    e = check_brackets(e)
    e = e.lstrip('.').lstrip(' ')
    e = re.sub(r'\([^\)]*\)', '', e)
    e = clean_up_string(e)
    e = re.sub(r'\(|\)', '', e)
    e = re.sub(r'\s+', ' ', e)
    return e.strip()


def grep_entities(filename):
    with open(filename, 'r', encoding='UTF-8') as rf:
        data = rf.read()
        txt = re.sub(r'\n', u"\u2063", data)
        
        par_pattern = r'<p>.*?<entity [^>]*>.*?</p>'
        match_pars = re.findall(par_pattern, txt)

        out_grep = dict()

        ent_pattern = r'<entity [^>]*>\s*(.*?)\s*<\/entity>'
        for par in match_pars:
            par_lines = par.split(u"\u2063")
            par_first = []
            langs = {'cs': 0, 'en': 0, 'de': 0, 'la': 0, 'unk': 0}

            for line in par_lines:
                if line.startswith('<') and line.endswith('>'):
                    par_first.append(line)
                else:
                    line_info = line.split('\t')
                    par_first.append(line_info[0])
                    langs[line_info[-1]] += 1

            par_first = ' '.join(par_first)
            match_ent = re.findall(ent_pattern, par_first)
            langs['unk'] = -1
            found_lang = max(langs, key=langs.get)
            found_ent = list()
            for ent in match_ent:
                found_ent.append(clean_text(ent))
            out_grep[max(out_grep) + 1 if out_grep else 0] = {'context': clean_text(par_first), 'entities': found_ent, 'lang': found_lang}
        return out_grep

def extract(filename, out_filename):
    out_grep = grep_entities(filename)
    out_file = open(out_filename, 'w', encoding='UTF-8')
    json.dump(out_grep, out_file)

extract(sys.argv[1], sys.argv[2])
    
