#!/usr/bin/env python3

import json
import sys
from tqdm import tqdm
import pdt_to_majka as p2m
import run_majka


def generate(filename):
    data = None
    with open(filename, 'r', encoding='UTF-8') as rf:
        data = json.load(rf)

    words_out = list()
    for c in tqdm(data):
         
        words = c['words']
        words = {int(k):v for k,v in words.items()}
        lang = c['lang']
        trees = c['trees']
        entities = c['original']
        types = c['types']
        ids = c['ids']

        for (tree, start, end) in trees:
            if tree:
                tree = {int(k):v for k,v in tree.items()}
                top = tree[min(tree.keys())][0]
                if words[top]['class'] in ['NOUN', 'ADJ', 'PROPN']:
                    words[top]['lemmatize'] = True
                    path = [top]
                else:
                    path = []
                
                while path:
                    for parent in path:
                        new_parents = []
                        kids = tree[parent] if str(parent) in tree else []
                        for kid in kids:
                            kid = str(kid)
                            if words[parent]['class'] == 'CCONJ':
                                words[kid]['lemmatize'] = True
                                new_parenst.append(kid)
                            elif words[kid]['class'] in ['NOUN', 'ADJ', 'PROPN'] and len(words[kid]['tags']) >= 5 and len(words[kid]['tags']) >= 5 and words[kid]['tags'][2:5] == words[parent]['tags'][2:5]:
                                words[kid]['lemmatize'] = True
                                new_parents.append(kid)
                            elif words[kid]['class'] == 'CCONJ':
                                new_parents.append(kid)
                        path = new_parents
            words_out.append('<context entity="{0}">'.format(entities.pop(0)))
            for k in words:
                if k == start + 1:
                    words_out.append('<entity{0}{1}>'.format(' type="{0}"'.format(types.pop(0) if types else ''), ' authId="{0}"'.format(ids.pop(0) if ids else '')))
                if k > max(start - 5, -1) and k <= min(end + 5, len(words)):
                    if lang == 'cs' and 'lemmatize' in words[k] and words[k]['class'] != 'CCONJ':
                        mtag = p2m.get_majka_tag(words[k]['tags'][:3] + 'S1' + words[k]['tags'][5:], words[k]['class'])
                        found = run_majka.find_form(words[k]['lemma'], mtag)
                        if found:
                            words[k]['lemma'] = found[0]['word']
                        words[k]['tags'] = words[k]['tags'][:4] + '1' + words[k]['tags'][5:]

                        if words[k]['word'][0].isupper():
                            words[k]['lemma'] = words[k]['lemma'][0].upper() + words[k]['lemma'][1:]
                    words_out.append('\t'.join([
                        words[k]['lemma'] if 'lemmatize' in words[k] else words[k]['word'],
                        words[k]['lemma'],
                        words[k]['tags']
                    ]))
                if k == end:
                    words_out.append('</entity>')
            words_out.append('</context>')
    return words_out

def run(filename, out_filename):
    output = ['<doc filename="{0}">'.format(filename.rsplit('/', 1)[-1].rsplit('.')[0])]
    output.extend(generate(filename))
    output.append('</doc>')
    of = open(out_filename, 'w', encoding='UTF-8')
    of.write('\n'.join(output))
    of.close()

run(sys.argv[1], sys.argv[2])
